﻿
namespace HeuristicLab.Problems.RoutePlanning.Graph {
  public class Edge<Vertex> : IEdge<Vertex> {
    private readonly Vertex source;
    public Vertex Source {
      get { return this.source; }
    }

    private readonly Vertex target;
    public Vertex Target {
      get { return this.target; }
    }

    private short category;
    public short Category {
      get { return category; }
      set { category = value; }
    }

    private float weight;
    public float Weight {
      get { return weight; }
      set { weight = value; }
    }

    public Edge(Vertex source, Vertex target)
      : this(source, target, 0) {

    }

    public Edge(Vertex source, Vertex target, short category) {
      this.source = source;
      this.target = target;
      this.category = category;
    }

    public override string ToString() {
      return string.Format("{0}->{1}", this.Source, this.Target);
    }

    public override bool Equals(object obj) {
      if (obj is Edge<Vertex>) {
        Edge<Vertex> e = (obj as Edge<Vertex>);
        return (this.source.Equals(e.source)) && (this.target.Equals(e.target));
      }
      return false;
    }

    public override int GetHashCode() {
      return source.GetHashCode() ^ target.GetHashCode();
    }

    public int GetMaxSpeed() {
      int speed = 0;

      //TODO: 
      switch (category) {
        case 23:
        case 17:
        case 18:
        case 22:
        case 21:
        case 24:
        case 0:
          speed = 1;
          break;

        case 20:
        case 15:
        case 12:
        case 11:
          speed = 15;
          break;

        case 19:
        case 16:
          speed = 30;
          break;

        case 9:
        case 10:
        case 7:
        case 8:
          speed = 80;
          break;

        case 14:
        case 13:
          speed = 50;
          break;

        case 3:
        case 4:
        case 5:
        case 6:
          speed = 100;
          break;

        case 1:
        case 2:
          speed = 130;
          break;

        default:
          speed = 1;
          break;
      }
      return speed;
    }
  }
}
